#include "sa.h"

void SA::GetSym(bool &errors)
{ Lex->getsym(SYM, errors); }


bool SA::match_operand(SA_unpackedlines &srcline, SA_addresses& addr, ADR_MODE am)
{
	addr.name[0] = '\0';
	if ( am == AM_x ) 
		return true;
	sprintf(addr.name, "%.*s", ASM_alength, SYM.str);
	if ( am == AM_AL && SYM.sym == LA_regsym && strcmp ( SYM.str, "AL" ) == 0 ) 
	{	addr.dsize = DS_BYTE;
		GetSym(srcline.errors);
		return true;
	}
	if ( am == AM_EAX && SYM.sym == LA_regsym && strcmp ( SYM.str, "EAX" ) == 0 ) 
	{	addr.dsize = DS_DWORD;
		GetSym(srcline.errors);
		return true;
	}
	if ( am == AM_EAX && SYM.sym == LA_regsym && strcmp ( SYM.str, "AX" ) == 0 ) 
	{	addr.dsize = DS_WORD;
		GetSym(srcline.errors);
		return true;
	}
	if ( am == AM_DX && SYM.sym == LA_regsym && strcmp ( SYM.str, "DX" ) == 0 ) 
	{	addr.dsize = DS_WORD;
		GetSym(srcline.errors);
		return true;
	}
	if ( am == AM_CL && SYM.sym == LA_regsym && strcmp ( SYM.str, "CL" ) == 0 ) 
	{	addr.dsize = DS_BYTE;
		GetSym(srcline.errors);
		return true;
	}
	if ( am == AM_r8 && SYM.sym == LA_regsym && SYM.num==8 ) 
	{	addr.dsize = DS_BYTE;
		srcline.reg=SYM.reg;
		GetSym(srcline.errors);
		return true;
	}
	if ( am == AM_r32 && SYM.sym == LA_regsym && SYM.num >= 16 ) 
	{	addr.dsize = (SYM.num==16)?DS_WORD:DS_DWORD;
		srcline.reg=SYM.reg;
		GetSym(srcline.errors);
		return true;
	}
    // immediate modes
	if ( am == AM_imm8 || am == AM_imm16 || am == AM_imm32 )
	{	
		if ( SYM.sym == LA_numsym )
		{	addr.dsize = (am==AM_imm8)?DS_BYTE:((am==AM_imm16)?DS_WORD:DS_DWORD);
			getaddress(addr, srcline.errors );
			return true;
		}
		if ( SYM.sym == LA_offset )
		{	GetSym(srcline.errors);
		    sprintf(addr.name, "%.*s", ASM_alength, SYM.str);
			if ( SYM.sym == LA_idsym )
			{
				getaddress(addr, srcline.errors);
				return true;
			}
			else
				return false;
		}
	}
	// relative modes
	if ( am == AM_rel32 && SYM.sym == LA_idsym ) // rel8 currently not supported
	{	
		addr.dsize = DS_DWORD;
		getaddress(addr, srcline.errors );
		return true;
	}
	// TODO process rm... modes
	if ( am==AM_rm8 || am==AM_rm32 )
	{   
		addr.dsize = (am==AM_rm8)?DS_BYTE:DS_DWORD;
		if ( SYM.sym == LA_idsym )
		{
			srcline.rm=5;
			srcline.mod=0;
			getaddress(addr, srcline.errors);
			return true;
		}
		if ( SYM.sym == LA_regsym )
		{
			srcline.rm=SYM.reg;
			srcline.mod=3;
			if ((am==AM_rm8 && SYM.num!=8) || (am!=AM_rm8 && SYM.num==8))
				return false;
			if (SYM.num==16)
				addr.dsize=DS_WORD;
			GetSym(srcline.errors);
			return true;
		}
	}
	return false;
	
}


bool SA::match_all_operands(SA_unpackedlines &srcline, const X86_instr& tmplate )
// Try to unpack the operands of line into srcline
// for a template instruction given in tmplate
// returns true in case of success
{
	GetSym(srcline.errors);
	if (!match_operand(srcline, srcline.address1, tmplate.am1))
		return false;
	if ( tmplate.am2 == AM_x )
		return true;
	if ( SYM.sym != LA_COMMA )
		return false;
	GetSym(srcline.errors);
	if (!match_operand(srcline, srcline.address2, tmplate.am2))
		return false;
	if ( tmplate.am3 == AM_x )
		return true;
	if ( SYM.sym != LA_COMMA )
		return false;
	GetSym(srcline.errors);
	if (!match_operand(srcline, srcline.address3, tmplate.am3))
		return false;
	return true;
}


void SA::getaddress(SA_addresses &address, bool &errors)
// Unpack the addressfield of line into srcline
{ //sprintf(address.name, "%.*s", ASM_alength, SYM.str);
  address.number = SYM.num;
  switch (SYM.sym)
  { case LA_numsym:
      address.kind = SA_numeric; break;
    case LA_idsym:
      address.kind = SA_alphameric; break;
  }
  GetSym(errors); // trailing comment or EOL
}


void SA::parse(SA_unpackedlines &srcline)
{ srcline.labfield[0] = '\0';
  srcline.tmplate = x86.null();
  srcline.comment[0] = '\0';
  srcline.errors = false;
  srcline.SIB = -1; // default is - not needed
  srcline.address3.kind = srcline.address2.kind = srcline.address1.kind = SA_absent;
  srcline.address3.number = srcline.address2.number = srcline.address1.number = 0;
  srcline.address3.name[0] = srcline.address2.name[0] = srcline.address1.name[0] = '\0'; 
  srcline.address3.OFFSET = srcline.address2.OFFSET = srcline.address1.OFFSET = false;
  srcline.address3.dsize = srcline.address2.dsize = srcline.address1.dsize = DS_NONE;
  GetSym(srcline.errors);       // first on line - opcode or label ?
  if (SYM.sym == LA_eofsym) { srcline.tmplate= x86.END(); return; }
  srcline.labelled = SYM.islabel;
  if (srcline.labelled)         // must look for the opcode
  { sprintf(srcline.labfield, "%.*s", ASM_alength, SYM.str);
    GetSym(srcline.errors);     // probably an opcode
  }
  if (SYM.sym == LA_idsym)      // has a mnemonic
  { 
#if 0
	  sprintf(srcline.tmplate->mnemonic, "%.*s", ASM_alength, SYM.str);
#endif
	  // DB
    Lex->save_pos();
	int tmpl = x86.findfirst( SYM.str );
	while ( X86::table[tmpl].opcode != AS_err && !match_all_operands(srcline, X86::table[tmpl]) )
	{
		tmpl = x86.findnext();
		Lex->go_back();
	}
    srcline.tmplate = tmpl;
#if 0
	GetSym(srcline.errors);     // possibly an address
    if (SYM.sym == LA_idsym || SYM.sym == LA_numsym)
      getaddress(srcline);
#endif
  }
  if (SYM.sym == LA_comsym || SYM.sym == LA_unknown)
  { strcpy(srcline.comment, SYM.str); GetSym(srcline.errors); }
  if (SYM.sym != LA_eolsym)     // spurious symbol
  { strcpy(srcline.comment, SYM.str); srcline.errors = true; }
  while (SYM.sym != LA_eolsym && SYM.sym != LA_eofsym) 
    GetSym(srcline.errors);     // consume garbage
  Lex->skipeol();
}

SA::SA(LA * L)
{ Lex = L; }
