#ifndef SH_H
#define SH_H

#include "misc.h"

const int linemax = 129;        // limit on source line length

class SH {
  public:
    FILE *lst; // listing file
    char ch;   // latest character read

    void nextch(void);
    // Returns ch as the next character on current source line, reading a new
    // line where necessary.  ch is returned as NUL if src is exhausted

    bool endline(void)             { return (charpos == linelength); }
    // Returns true when end of current line has been reached

    bool startline(void)           { return (charpos == 1); }
    // Returns true if current ch is the first on a line

	void save_pos(void) { savedpos = charpos; }

	void go_back(void) 
	{  if ( savedpos != -1 ) 
	      charpos = savedpos;
	      ch = line[charpos-1];
	}

	void writehex(int i, int n)    { fprintf(lst, "%02X%*c", i, n-2, ' '); }
    // Writes (byte valued) i to lst file as hex pair, left-justified in n spaces

	void writeadr(int i, int n)    { fprintf(lst, "%04X%*c", i, n-4, ' '); }
    // Writes (word valued) i to lst file as hex pair, left-justified in n spaces

	void writetext(const char *s, int n) { fprintf(lst, "%-*s", n, s); }
    // Writes s to lst file left-justified in n spaces

    SH();
    // Default constructor

    SH(char *sourcename, char *listname, char *version);
    // Initializes source handler, and displays version information on lst file.
    // Opens src and lst files using given names

    ~SH();
    // Closes src and lst files

  private:
    FILE *src;              // source file
    int charpos;            // character pointer
	int savedpos;           // saved position within the current line
    int linelength;         // line length
    char line[linemax + 1]; // last line read
};

#endif /*SH_H*/
