#include "x86.h"
#include <string.h>

X86::X86() {
	curr = 0;
}


int X86::illegal() {
	return 0;
}

int X86::null() {
	return 1;
}

int X86::END() {
	return findfirst("END");
}


int X86::findfirst(char *mnemonic)
{
	curr=1;
	for ( curr=2; table[curr].mnemonic[0] != '\0'; curr++)
	{
		if ( strcmp( mnemonic, table[curr].mnemonic ) == 0 )
			return curr;
	}
	curr=0;
	return curr;
}


int X86::findnext()
{
	curr++;
	if ( table[curr].mnemonic[0] == '\0' || strcmp( table[curr-1].mnemonic, table[curr].mnemonic ) != 0 )
		curr = 0;
	
	return curr; 
}

bool X86::hasImm32Opnd( int tmpl ) {
    return table[tmpl].am1 == AM_imm32 || table[tmpl].am2 == AM_imm32 || table[tmpl].am3 == AM_imm32;  
}

const X86_instr X86::table [] = 
	{
		{0, AS_err, -1, false, -1, "Error", AM_x, AM_x, AM_x },
		{0, AS_nul, -1, false, -1, "", AM_x, AM_x, AM_x },
		{1, 0x37, -1  , false,-1, "AAA"      , AM_x, AM_x, AM_x }, //ASCII adjust AL after addition
		{2, 0xD5, 0x0A, false,-1, "AAD"      , AM_x, AM_x, AM_x }, //ASCII adjust AX before division
		{2, 0xD4, 0x0A, false,-1, "AAM"      , AM_x, AM_x, AM_x }, //ASCII adjust AX after multiplication
		{1, 0x3F, -1  , false,-1, "AAS"      , AM_x, AM_x, AM_x }, //ASCII adjust AL after subtraction
		{2, 0x14, -1  , false,-1, "ADC"      , AM_AL,AM_imm8, AM_x        }, //Add with carry 
		{5, 0x15, -1  , false,-1, "ADC"      , AM_EAX,AM_imm32, AM_x      }, //Add with carry 
		{3, 0x80, -1  , false, 2, "ADC"      , AM_rm8,AM_imm8, AM_x      }, //Add with carry 
		{6, 0x81, -1  , false, 2, "ADC"      , AM_rm32,AM_imm32, AM_x    }, //Add with carry
		{3, 0x83, -1  , false, 2, "ADC"      , AM_rm32,AM_imm8, AM_x     }, //Add with carry
		{2, 0x10, -1  , false, 8, "ADC"      , AM_rm8,AM_r8 , AM_x        }, //Add with carry 
		{2, 0x11, -1  , false, 8, "ADC"      , AM_rm32,AM_r32 , AM_x      }, //Add with carry
		{2, 0x12, -1  , false, 8, "ADC"      , AM_r8,AM_rm8 , AM_x        }, //Add with carry 
		{2, 0x13, -1  , false, 8, "ADC"      , AM_r32,AM_rm32 , AM_x      }, //Add with carry
		{2, 0x04, -1  , false,-1, "ADD"      , AM_AL,AM_imm8, AM_x        }, //Add 
		{5, 0x05, -1  , false,-1, "ADD"      , AM_EAX,AM_imm32, AM_x      }, //Add 
		{3, 0x80, -1  , false, 0, "ADD"      , AM_rm8,AM_imm8, AM_x      }, //Add 
		{6, 0x81, -1  , false, 0, "ADD"      , AM_rm32,AM_imm32, AM_x    }, //Add 
		{3, 0x83, -1  , false, 0, "ADD"      , AM_rm32,AM_imm8, AM_x     }, //Add 
		{2, 0x00, -1  , false, 8, "ADD"      , AM_rm8,AM_r8 , AM_x        }, //ADD 
		{2, 0x01, -1  , false, 8, "ADD"      , AM_rm32,AM_r32 , AM_x      }, //ADD 
		{2, 0x02, -1  , false, 8, "ADD"      , AM_r8,AM_rm8 , AM_x        }, //ADD 
		{2, 0x03, -1  , false, 8, "ADD"      , AM_r32,AM_rm32 , AM_x      }, //ADD 
		{2, 0x24, -1  , false,-1, "AND"      , AM_AL,AM_imm8, AM_x        }, //AND 
		{5, 0x25, -1  , false,-1, "AND"      , AM_EAX,AM_imm32, AM_x      }, //AND
		{3, 0x80, -1  , false, 4, "AND"      , AM_rm8,AM_imm8, AM_x      }, //AND
		{6, 0x81, -1  , false, 4, "AND"      , AM_rm32,AM_imm32, AM_x    }, //AND
		{3, 0x83, -1  , false, 4, "AND"      , AM_rm32,AM_imm8, AM_x     }, //AND
		{2, 0x20, -1  , false, 8, "AND"      , AM_rm8,AM_r8 , AM_x        }, //AND
		{2, 0x21, -1  , false, 8, "AND"      , AM_rm32,AM_r32 , AM_x      }, //AND
		{2, 0x22, -1  , false, 8, "AND"      , AM_r8,AM_rm8 , AM_x        }, //AND
		{2, 0x23, -1  , false, 8, "AND"      , AM_r32,AM_rm32 , AM_x      }, //AND
		//{1, 0x62, -1  , false, 8, "BOUND"    , AM_r32,m32&32, AM_x      }, //Check Array Index Against Bounds
		{0, AS_beg, -1, false, -1, "BEG", AM_x, AM_x, AM_x },
		{3, 0x0F, 0xBC, false, 8, "BSF"      , AM_r32,AM_rm32 , AM_x      }, //Bit scan forward on AM_rm32
		{3, 0x0F, 0xBD, false, 8, "BSR"      , AM_r32,AM_rm32 , AM_x      }, //Bit scan reverse on AM_rm32
		{2, 0x0F, 0xC8, true ,-1, "BSWAP"    , AM_r32 , AM_x, AM_x      }, //Reverses the byte order of a AM_r32
		{3, 0x0F, 0xA3, false, 8, "BT"       , AM_rm32,AM_r32 , AM_x      }, //Bit Test
		{4, 0x0F, 0xBA, false, 4, "BT"       , AM_rm32,AM_imm8, AM_x      }, //Bit Test
		{3, 0x0F, 0xBB, false, 8, "BTC"      , AM_rm32,AM_r32 , AM_x      }, //Bit Test and Complement
		{4, 0x0F, 0xBA, false, 7, "BTC"      , AM_rm32,AM_imm8, AM_x      }, //Bit Test and Complement
		{3, 0x0F, 0xB3, false, 8, "BTR"      , AM_rm32,AM_r32 , AM_x      }, //Bit Test and Clear
		{4, 0x0F, 0xBA, false, 6, "BTR"      , AM_rm32,AM_imm8, AM_x      }, //Bit Test and Clear
		{3, 0x0F, 0xAB, false, 8, "BTS"      , AM_rm32,AM_r32 , AM_x      }, //Bit Test and Set
		{4, 0x0F, 0xBA, false, 5, "BTS"      , AM_rm32,AM_imm8, AM_x      }, //Bit Test and Set
		{5, 0xE8, -1  , false,-1, "CALL"     , AM_rel32, AM_x, AM_x     }, //Call near, rel to n.inst
		{2, 0xFF, -1  , false, 2, "CALL"     , AM_rm32 , AM_x, AM_x    }, //Call near, abs.ind.add. given in AM_rm32
		//{1, 0x9A, -1  , false,-1, "CALL"     , ptr16:32, AM_x, AM_x  }, //Call far, abs.add. given in operand
		//{1, 0xFF, -1  , false, 3, "CALL"     , m16:32, AM_x, AM_x    }, //Call far, abs.ind.add. given in m16:32
		{1, 0x98, -1  , false,-1, "CBW"      , AM_x, AM_x, AM_x }, //Convert Byte to Word
		{1, 0x99, -1  , false,-1, "CWD"      , AM_x, AM_x, AM_x }, //Convert Word to Doubleword
		{1, 0x99, -1  , false,-1, "CDQ"      , AM_x, AM_x, AM_x }, //Convert Doubleword to Quadword 
		{1, 0xF8, -1  , false,-1, "CLC"      , AM_x, AM_x, AM_x }, //Clear CF flag
		{1, 0xFC, -1  , false,-1, "CLD"      , AM_x, AM_x, AM_x }, //Clear DF flag
		{1, 0xFA, -1  , false,-1, "CLI"      , AM_x, AM_x, AM_x }, //Clear interrupt flag
		{1, 0xF5, -1  , false,-1, "CMC"      , AM_x, AM_x, AM_x }, //Complement CF flag
		{3, 0x0F, 0x47, false, 8, "CMOVA"    , AM_r32,AM_rm32 , AM_x      }, //Move if above 
		{3, 0x0F, 0x43, false, 8, "CMOVAE"   , AM_r32,AM_rm32 , AM_x      }, //Move if above or equal 
		{3, 0x0F, 0x42, false, 8, "CMOVB"    , AM_r32,AM_rm32 , AM_x      }, //Move if below 
		{3, 0x0F, 0x46, false, 8, "CMOVBE"   , AM_r32,AM_rm32 , AM_x      }, //Move if below or equal 
		{3, 0x0F, 0x42, false, 8, "CMOVC"    , AM_r32,AM_rm32 , AM_x      }, //Move if carry 
		{3, 0x0F, 0x44, false, 8, "CMOVE"    , AM_r32,AM_rm32 , AM_x      }, //Move if equal 
		{3, 0x0F, 0x4F, false, 8, "CMOVG"    , AM_r32,AM_rm32 , AM_x      }, //Move if greater 
		{3, 0x0F, 0x4D, false, 8, "CMOVGE"   , AM_r32,AM_rm32 , AM_x      }, //Move if greater or equal 
		{3, 0x0F, 0x4C, false, 8, "CMOVL"    , AM_r32,AM_rm32 , AM_x      }, //Move if less 
		{3, 0x0F, 0x4E, false, 8, "CMOVLE"   , AM_r32,AM_rm32 , AM_x      }, //Move if less or equal 
		{3, 0x0F, 0x46, false, 8, "CMOVNA"   , AM_r32,AM_rm32 , AM_x      }, //Move if not above 
		{3, 0x0F, 0x42, false, 8, "CMOVNAE"  , AM_r32,AM_rm32 , AM_x      }, //Move if not above or equal 
		{3, 0x0F, 0x43, false, 8, "CMOVNB"   , AM_r32,AM_rm32 , AM_x      }, //Move if not below 
		{3, 0x0F, 0x47, false, 8, "CMOVNBE"  , AM_r32,AM_rm32 , AM_x      }, //Move if not below or equal 
		{3, 0x0F, 0x43, false, 8, "CMOVNC"   , AM_r32,AM_rm32 , AM_x      }, //Move if not carry 
		{3, 0x0F, 0x45, false, 8, "CMOVNE"   , AM_r32,AM_rm32 , AM_x      }, //Move if not equal 
		{3, 0x0F, 0x4E, false, 8, "CMOVNG"   , AM_r32,AM_rm32 , AM_x      }, //Move if not greater 
		{3, 0x0F, 0x4C, false, 8, "CMOVNGE"  , AM_r32,AM_rm32 , AM_x      }, //Move if not greater or equal 
		{3, 0x0F, 0x4D, false, 8, "CMOVNL"   , AM_r32,AM_rm32 , AM_x      }, //Move if not less 
		{3, 0x0F, 0x4F, false, 8, "CMOVNLE"  , AM_r32,AM_rm32 , AM_x      }, //Move if not less or equal 
		{3, 0x0F, 0x41, false, 8, "CMOVNO"   , AM_r32,AM_rm32 , AM_x      }, //Move if not overflow 
		{3, 0x0F, 0x4B, false, 8, "CMOVNP"   , AM_r32,AM_rm32 , AM_x      }, //Move if not parity 
		{3, 0x0F, 0x49, false, 8, "CMOVNS"   , AM_r32,AM_rm32 , AM_x      }, //Move if not sign 
		{3, 0x0F, 0x45, false, 8, "CMOVNZ"   , AM_r32,AM_rm32 , AM_x      }, //Move if not zero 
		{3, 0x0F, 0x40, false, 8, "CMOVO"    , AM_r32,AM_rm32 , AM_x      }, //Move if overflow 
		{3, 0x0F, 0x4A, false, 8, "CMOVP"    , AM_r32,AM_rm32 , AM_x      }, //Move if parity 
		{3, 0x0F, 0x4A, false, 8, "CMOVPE"   , AM_r32,AM_rm32 , AM_x      }, //Move if parity even 
		{3, 0x0F, 0x4B, false, 8, "CMOVPO"   , AM_r32,AM_rm32 , AM_x      }, //Move if parity odd 
		{3, 0x0F, 0x48, false, 8, "CMOVS"    , AM_r32,AM_rm32 , AM_x      }, //Move if sign 
		{3, 0x0F, 0x44, false, 8, "CMOVZ"    , AM_r32,AM_rm32 , AM_x      }, //Move if zero 
		{2, 0x3C, -1  , false,-1, "CMP"      , AM_AL,AM_imm8, AM_x        }, //Compare 
		{5, 0x3D, -1  , false,-1, "CMP"      , AM_EAX,AM_imm32, AM_x      }, //Compare 
		{3, 0x80, -1  , false, 7, "CMP"      , AM_rm8,AM_imm8, AM_x      }, //Compare 
		{6, 0x81, -1  , false, 7, "CMP"      , AM_rm32,AM_imm32, AM_x    }, //Compare 
		{3, 0x83, -1  , false, 7, "CMP"      , AM_rm32,AM_imm8, AM_x     }, //Compare 
		{2, 0x38, -1  , false, 8, "CMP"      , AM_rm8,AM_r8 , AM_x        }, //Compare 
		{2, 0x39, -1  , false, 8, "CMP"      , AM_rm32,AM_r32 , AM_x      }, //Compare 
		{2, 0x3A, -1  , false, 8, "CMP"      , AM_r8,AM_rm8 , AM_x        }, //Compare 
		{2, 0x3B, -1  , false, 8, "CMP"      , AM_r32,AM_rm32 , AM_x      }, //Compare 
		{1, 0xA6, -1  , false,-1, "CMPSB"    , AM_x, AM_x, AM_x }, //Compare byte at DS:(E)SI with ES:(E)DI 
		{1, 0xA7, -1  , false,-1, "CMPSD"    , AM_x, AM_x, AM_x }, //Compare dw   at DS:(E)SI with ES:(E)DI
		{3, 0x0F, 0xB0, false, 8, "CMPXCHG"  , AM_rm8,AM_r8 , AM_x        }, //Compare and Exchange
		{3, 0x0F, 0xB1, false, 8, "CMPXCHG"  , AM_rm32,AM_r32 , AM_x      }, //Compare and Exchange
		//{1, 0x0F, 0xC7, false, 1, "CMPXCHG8B", AM_m64, AM_x, AM_x       }, //Compare and Exchange
		{2, 0x0F, 0xA2, false,-1, "CPUID"    , AM_x, AM_x, AM_x }, //EAX := Processor id.info.
		{1, 0x27, -1  , false,-1, "DAA"      , AM_x, AM_x, AM_x }, //Decimal adjust AL after addition
		{1, 0x2F, -1  , false,-1, "DAS"      , AM_x, AM_x, AM_x }, //Decimal adjust AL after subtraction
		{1, AS_dc, -1, false, -1, "DC", AM_imm8, AM_x, AM_x },
		{2, 0xFE, -1  , false, 1, "DEC"      , AM_rm8 , AM_x, AM_x     }, //Decrement AM_rm8 , AM_xby 1
		{2, 0xFF, -1  , false, 1, "DEC"      , AM_rm32 , AM_x, AM_x    }, //Decrement AM_rm32 , AM_xby 1
		{1, 0x48, -1  , true ,-1, "DEC"      , AM_r32 , AM_x, AM_x      }, //Decrement AM_r32 , AM_xby 1
		{2, 0xF6, -1  , false, 6, "DIV"      , AM_rm8 , AM_x, AM_x     }, //Unsigned divide AX by AM_rm8
		//{1, 0xF7, -1  , false, 6, "DIV"      , r/m16, AM_x, AM_x     }, //Unsigned divide DX:AX by r/m16
		{2, 0xF7, -1  , false, 6, "DIV"      , AM_rm32 , AM_x, AM_x    }, //Unsigned divide EDX:EAX by AM_rm32 , AM_x
		{0, AS_ds, -1, false, -1, "DS", AM_imm8, AM_x, AM_x },
		//{1, 0xC8, -1  , false,-1, "ENTER"    , AM_imm16,0, AM_x         }, //Create a stack frame for a procedure
		//{1, 0xC8, -1  , false,-1, "ENTER"    , AM_imm16,1, AM_x         }, //Create a nested stack frame for a proc.
		{0, AS_end, -1, false, -1, "END", AM_x, AM_x, AM_x },
		{4, 0xC8, -1  , false,-1, "ENTER"    , AM_imm16,AM_imm8, AM_x     }, //Create a nested stack frame for a proc.
		{0, AS_equ, -1, false, -1, "EQU", AM_imm32, AM_x, AM_x },
		{1, 0xF4, -1  , false,-1, "HLT"      , AM_x, AM_x, AM_x }, //Halt
		{2, 0xF6, -1  , false, 7, "IDIV"     , AM_rm8 , AM_x, AM_x     }, //Divide   
		{2, 0xF7, -1  , false, 7, "IDIV"     , AM_rm32 , AM_x, AM_x    }, //Divide  
		{2, 0xF6, -1  , false, 5, "IMUL"     , AM_rm8 , AM_x, AM_x     }, //Multiply
		{2, 0xF7, -1  , false, 5, "IMUL"     , AM_rm32 , AM_x, AM_x    }, //Multiply
		{3, 0x0F, 0xAF, false, 8, "IMUL"     , AM_r32,AM_rm32 , AM_x      }, //Multiply
		{3, 0x6B, -1  , false, 8, "IMUL"     , AM_r32,AM_rm32,AM_imm8,       }, //Multiply
		{3, 0x6B, -1  , false, 8, "IMUL"     , AM_r32,AM_imm8, AM_x       }, //Multiply
		{6, 0x69, -1  , false, 8, "IMUL"     , AM_r32,AM_rm32,AM_imm32,     }, //Multiply
		{6, 0x69, -1  , false, 8, "IMUL"     , AM_r32,AM_imm32, AM_x      }, //Multiply
		{2, 0xE4, -1  , false,-1, "IN"       , AM_AL,AM_imm8, AM_x        }, //Input byte from AM_imm8, AM_xI/O port address into AL
		{2, 0xE5, -1  , false,-1, "IN"       , AM_EAX,AM_imm8, AM_x       }, //Input byte from AM_imm8, AM_xI/O port address into EAX
		{1, 0xEC, -1  , false,-1, "IN"       , AM_AL,AM_DX, AM_x           }, //Input byte from I/O port in DX into AL
		{1, 0xED, -1  , false,-1, "IN"       , AM_EAX,AM_DX, AM_x          }, //Input doubleword from I/O port in DX into EAX
		{2, 0xFE, -1  , false, 0, "INC"      , AM_rm8 , AM_x, AM_x     }, //Increment 1
		{2, 0xFF, -1  , false, 0, "INC"      , AM_rm32 , AM_x, AM_x    }, //Increment 1
		{1, 0x40, -1  , true ,-1, "INC"      , AM_r32 , AM_x, AM_x      }, //Increment register by 1
		{2, 0x77, -1  , false,-1, "JA"       , AM_rel8, AM_x, AM_x           }, //Jump short if above 
		{6, 0x0F, 0x87, false,-1, "JA"       , AM_rel32, AM_x, AM_x          }, //Jump near if above 
		{2, 0x73, -1  , false,-1, "JAE"      , AM_rel8, AM_x, AM_x           }, //Jump short if above or equal 
		{6, 0x0F, 0x83, false,-1, "JAE"      , AM_rel32, AM_x, AM_x          }, //Jump near if above or equal 
		{2, 0x72, -1  , false,-1, "JB"       , AM_rel8, AM_x, AM_x          }, //Jump near if below 
		{6, 0x0F, 0x82, false,-1, "JB"       , AM_rel32, AM_x, AM_x          }, //Jump near if below 
		{2, 0x76, -1  , false,-1, "JBE"      , AM_rel8, AM_x, AM_x           }, //Jump short if below or equal 
		{6, 0x0F, 0x86, false,-1, "JBE"      , AM_rel32, AM_x, AM_x          }, //Jump near if below or equal 
		{2, 0x72, -1  , false,-1, "JC"       , AM_rel8, AM_x, AM_x           }, //Jump short if carry 
		{6, 0x0F, 0x82, false,-1, "JC"       , AM_rel32, AM_x, AM_x           }, //Jump short if carry 
		{3, 0x66, 0xE3, false,-1, "JCXZ"    , AM_rel32 /* in fact, rel 8*/, AM_x, AM_x           }, //Jump short if ECX register is 0
		{2, 0x74, -1  , false,-1, "JE"       , AM_rel8, AM_x, AM_x           }, //Jump short if equal 
		{6, 0x0F, 0x84, false,-1, "JE"       , AM_rel32, AM_x, AM_x          }, //Jump near if equal 
		{2, 0xE3, -1  , false,-1, "JECXZ"    , AM_rel32/* in fact, rel 8*/, AM_x, AM_x           }, //Jump short if ECX register is 0
		{2, 0x7F, -1  , false,-1, "JG"       , AM_rel8, AM_x, AM_x           }, //Jump short if greater 
		{6, 0x0F, 0x8F, false,-1, "JG"       , AM_rel32, AM_x, AM_x          }, //Jump near if greater 
		{2, 0x7D, -1  , false,-1, "JGE"      , AM_rel8, AM_x, AM_x           }, //Jump short if greater or equal 
		{6, 0x0F, 0x8D, false,-1, "JGE"      , AM_rel32, AM_x, AM_x          }, //Jump near if greater or equal 
		{2, 0x7C, -1  , false,-1, "JL"       , AM_rel8, AM_x, AM_x           }, //Jump short if less 
		{6, 0x0F, 0x8C, false,-1, "JL"       , AM_rel32, AM_x, AM_x          }, //Jump near if less 
		{2, 0x7E, -1  , false,-1, "JLE"      , AM_rel8, AM_x, AM_x           }, //Jump short if less or equal 
		{6, 0x0F, 0x8E, false,-1, "JLE"      , AM_rel32, AM_x, AM_x          }, //Jump near if less or equal 
		{2, 0x72, -1  , false,-1, "JNAE"     , AM_rel8, AM_x, AM_x          }, //Jump near if below 
		{6, 0x0F, 0x82, false,-1, "JNAE"     , AM_rel32, AM_x, AM_x          }, //Jump near if below 
		{2, 0x73, -1  , false,-1, "JNB"      , AM_rel8, AM_x, AM_x           }, //Jump short if above or equal 
		{6, 0x0F, 0x83, false,-1, "JNB"      , AM_rel32, AM_x, AM_x          }, //Jump near if above or equal 
		{2, 0x77, -1  , false,-1, "JNBE"     , AM_rel8, AM_x, AM_x           }, //Jump short if above 
		{6, 0x0F, 0x87, false,-1, "JNBE"     , AM_rel32, AM_x, AM_x          }, //Jump near if above 
		{2, 0x73, -1  , false,-1, "JNC"      , AM_rel8, AM_x, AM_x           }, //Jump short if above or equal 
		{6, 0x0F, 0x83, false,-1, "JNC"      , AM_rel32, AM_x, AM_x          }, //Jump near if above or equal 
		{2, 0x75, -1  , false,-1, "JNE"      , AM_rel8, AM_x, AM_x           }, //Jump short if not equal 
		{6, 0x0F, 0x85, false,-1, "JNE"      , AM_rel32, AM_x, AM_x          }, //Jump near if not equal 
		{2, 0x7E, -1  , false,-1, "JNG"      , AM_rel8, AM_x, AM_x           }, //Jump short if less or equal 
		{6, 0x0F, 0x8E, false,-1, "JNG"      , AM_rel32, AM_x, AM_x          }, //Jump near if less or equal 
		{2, 0x7C, -1  , false,-1, "JNGE"     , AM_rel8, AM_x, AM_x           }, //Jump short if less 
		{6, 0x0F, 0x8C, false,-1, "JNGE"     , AM_rel32, AM_x, AM_x          }, //Jump near if less 
		{2, 0x7D, -1  , false,-1, "JNL"      , AM_rel8, AM_x, AM_x           }, //Jump short if greater or equal 
		{6, 0x0F, 0x8D, false,-1, "JNL"      , AM_rel32, AM_x, AM_x          }, //Jump near if greater or equal 
		{2, 0x7F, -1  , false,-1, "JNLE"     , AM_rel8, AM_x, AM_x           }, //Jump short if greater 
		{6, 0x0F, 0x8F, false,-1, "JNLE"     , AM_rel32, AM_x, AM_x          }, //Jump near if greater 
		{2, 0x71, -1  , false,-1, "JNO"      , AM_rel8, AM_x, AM_x           }, //Jump short if not overflow 
		{6, 0x0F, 0x81, false,-1, "JNO"      , AM_rel32, AM_x, AM_x          }, //Jump near if not overflow 
		{2, 0x7B, -1  , false,-1, "JNP"      , AM_rel8, AM_x, AM_x           }, //Jump short if parity odd 
		{6, 0x0F, 0x8B, false,-1, "JNP"      , AM_rel32, AM_x, AM_x          }, //Jump near if parity odd 
		{2, 0x79, -1  , false,-1, "JNS"      , AM_rel8, AM_x, AM_x           }, //Jump short if not sign 
		{6, 0x0F, 0x89, false,-1, "JNS"      , AM_rel32, AM_x, AM_x          }, //Jump near if not sign 
		{2, 0x75, -1  , false,-1, "JNZ"      , AM_rel8, AM_x, AM_x           }, //Jump short if not equal 
		{6, 0x0F, 0x85, false,-1, "JNZ"      , AM_rel32, AM_x, AM_x          }, //Jump near if not equal 
		{2, 0x70, -1  , false,-1, "JO"       , AM_rel8, AM_x, AM_x           }, //Jump short if overflow 
		{6, 0x0F, 0x80, false,-1, "JO"       , AM_rel32, AM_x, AM_x          }, //Jump near if overflow 
		{2, 0x7A, -1  , false,-1, "JP"      , AM_rel8, AM_x, AM_x           }, //Jump short if parity even 
		{6, 0x0F, 0x8A, false,-1, "JP"      , AM_rel32, AM_x, AM_x          }, //Jump near if parity even 
		{2, 0x7A, -1  , false,-1, "JPE"      , AM_rel8, AM_x, AM_x           }, //Jump short if parity even 
		{6, 0x0F, 0x8A, false,-1, "JPE"      , AM_rel32, AM_x, AM_x          }, //Jump near if parity even 
		{2, 0x7B, -1  , false,-1, "JPO"      , AM_rel8, AM_x, AM_x           }, //Jump short if parity odd 
		{6, 0x0F, 0x8B, false,-1, "JPO"      , AM_rel32, AM_x, AM_x          }, //Jump near if parity odd 
		{2, 0x78, -1  , false,-1, "JS"       , AM_rel8, AM_x, AM_x           }, //Jump short if sign 
		{6, 0x0F, 0x88, false,-1, "JS"       , AM_rel32, AM_x, AM_x          }, //Jump near if sign 
		{2, 0xEB, -1  , false,-1, "JMP"      , AM_rel8, AM_x, AM_x           }, //Jump short, relative, 
		{6, 0xE9, -1  , false,-1, "JMP"      , AM_rel32, AM_x, AM_x          }, //Jump near, relative, 
		{2, 0xFF, -1  , false, 4, "JMP"      , AM_rm32 , AM_x, AM_x         }, //Jump near, abs.ind.in AM_rm32
		//{1, 0xEA, -1  , false,-1, "JMP"      , ptr16:32, AM_x, AM_x       }, //Jump far, abs.add given in operand
		//{1, 0xFF, -1  , false, 8, "JMP"      , m16:32, AM_x, AM_x         }, //Jump far, abs.ind.in m16:32
		{2, 0x74, -1  , false,-1, "JZ"       , AM_rel8, AM_x, AM_x           }, //Jump short if equal 
		{6, 0x0F, 0x84, false,-1, "JZ"       , AM_rel32, AM_x, AM_x           }, //Jump short if equal 
		{1, 0x9F, -1  , false,-1, "LAHF"     , AM_x, AM_x, AM_x }, //Load Status Flags into AH 
		//{1, 0xC5, -1  , false, 8, "LDS"      , AM_r32,m16:32, AM_x      }, //Load DS:AM_r32 , AM_xwith far ptr
		//{1, 0x8D, -1  , false, 8, "LEA"      , AM_r32,m, AM_x           }, //Load effective address  
		{1, 0xC9, -1  , false,-1, "LEAVE"    , AM_x, AM_x, AM_x }, //Set ESP to EBP, then pop EBP
		{1, 0xAC, -1  , false,-1, "LODSB"     , AM_x, AM_x, AM_x         }, //Load byte at address DS:(E)SI into AL
		{1, 0xAD, -1  , false,-1, "LODSD"     , AM_x, AM_x, AM_x           }, //Load dword at address DS:(E)SI into EAX
		{2, 0x66,0xAD , false,-1, "LODSW"     , AM_x, AM_x, AM_x           }, //Load word at address DS:(E)SI into AX
		{2, 0xE2, -1  , false,-1, "LOOP"     , AM_rel8, AM_x, AM_x           }, //Dec count;jump if count # 0
		{2, 0xE1, -1  , false,-1, "LOOPE"    , AM_rel8, AM_x, AM_x           }, //Dec count;jump if count # 0 and ZF=1
		{2, 0xE1, -1  , false,-1, "LOOPZ"    , AM_rel8, AM_x, AM_x           }, //Dec count;jump if count # 0 and ZF=1
		{2, 0xE0, -1  , false,-1, "LOOPNE"   , AM_rel8, AM_x, AM_x           }, //Dec count;jump if count # 0 and ZF=0
		{2, 0xE0, -1  , false,-1, "LOOPNZ"   , AM_rel8, AM_x, AM_x           }, //Dec count;jump if count # 0 and ZF=0
		{2, 0x88, -1  , false, 8, "MOV"      , AM_rm8,AM_r8 , AM_x        }, //Move 
		{2, 0x89, -1  , false, 8, "MOV"      , AM_rm32,AM_r32 , AM_x      }, //Move 
		{2, 0x8A, -1  , false, 8, "MOV"      , AM_r8,AM_rm8 , AM_x        }, //Move 
		{2, 0x8B, -1  , false, 8, "MOV"      , AM_r32,AM_rm32 , AM_x      }, //Move 
		//{1, 0xA0, -1  , false,-1, "MOV"      , AM_AL, moffs8, AM_x     }, //Move byte at ( seg:offset) to AL
		//{1, 0xA1, -1  , false,-1, "MOV"      , AX, moffs16, AM_x    }, //Move word at ( seg:offset) to AX
		//{1, 0xA1, -1  , false,-1, "MOV"      , AM_EAX, moffs32, AM_x   }, //Move dword at ( seg:offset) to EAX
		//{1, 0xA2, -1  , false,-1, "MOV"      , moffs8,AM_AL, AM_x      }, //Move AL to ( seg:offset)
		//{1, 0xA3, -1  , false,-1, "MOV"      , moffs16,AX, AM_x     }, //Move AX to ( seg:offset)
		//{1, 0xA3, -1  , false,-1, "MOV"      , moffs32,AM_EAX, AM_x    }, //Move EAX to ( seg:offset)
		{2, 0xB0, -1  , true ,-1, "MOV"      , AM_r8,AM_imm8, AM_x        }, //Move AM_imm8, AM_xto AM_r8
		{5, 0xB8, -1  , true ,-1, "MOV"      , AM_r32,AM_imm32, AM_x      }, //Move AM_imm32, AM_xto AM_r32
		{3, 0xC6, -1  , false, 0, "MOV"      , AM_rm8,AM_imm8, AM_x      }, //Move AM_imm8, AM_xto AM_rm8
		{6, 0xC7, -1  , false, 0, "MOV"      , AM_rm32,AM_imm32, AM_x    }, //Move AM_imm32, AM_xto AM_rm32
		{1, 0xA4, -1  , false,-1, "MOVSB"     , AM_x,AM_x, AM_x           }, //Move byte at DS:(E)SI to  ES:(E)DI
		{1, 0xA5, -1  , false,-1, "MOVSD"     , AM_x,AM_x, AM_x         }, //Move dword at DS:(E)SI to  ES:(E)DI
		{2, 0x66, 0xA5, false,-1, "MOVSW"     , AM_x,AM_x, AM_x         }, //Move dword at DS:(E)SI to  ES:(E)DI
		{3, 0x0F, 0xBE, false, 8, "MOVSX"    , AM_r32,AM_rm8 , AM_x       }, //Move byte to doubleword, sign-extension
		//{1, 0x0F, 0xBF, false, 8, "MOVSX"    , AM_r32,r/m16, AM_x       }, //Move word to doubleword, sign-extension
		{3, 0x0F, 0xB6, false, 8, "MOVZX"    , AM_r32,AM_rm8 , AM_x       }, //Move byte to doubleword, zero-extension
		//{1, 0x0F, 0xB7, false, 8, "MOVZX"    , AM_r32,r/m16, AM_x       }, //Move word to doubleword, zero-extension
		{2, 0xF6, -1  , false, 4, "MUL"      , AM_rm8 , AM_x, AM_x           }, //Unsigned multiply 
		{2, 0xF7, -1  , false, 4, "MUL"      , AM_rm32 , AM_x, AM_x          }, //Unsigned multiply 
		{2, 0xF6, -1  , false, 3, "NEG"      , AM_rm8 , AM_x, AM_x           }, //Two's complement negate AM_rm8
		{2, 0xF7, -1  , false, 3, "NEG"      , AM_rm32 , AM_x, AM_x          }, //Two's complement negate AM_rm32
		{1, 0x90, -1  , false,-1, "NOP"      , AM_x, AM_x, AM_x }, //No operation
		{2, 0xF6, -1  , false, 2, "NOT"      , AM_rm8 , AM_x, AM_x           }, //Reverse each bit of AM_rm8
		{2, 0xF7, -1  , false, 2, "NOT"      , AM_rm32 , AM_x, AM_x          }, //Reverse each bit of AM_rm32
		{2, 0x0C, -1  , false,-1, "OR"       , AM_AL,AM_imm8, AM_x        }, //OR
		{5, 0x0D, -1  , false,-1, "OR"       , AM_EAX,AM_imm32, AM_x      }, //OR 
		{3, 0x80, -1  , false, 1, "OR"       , AM_rm8,AM_imm8, AM_x      }, //OR 
		{6, 0x81, -1  , false, 1, "OR"       , AM_rm32,AM_imm32, AM_x    }, //OR 
		{3, 0x83, -1  , false, 1, "OR"       , AM_rm32,AM_imm8, AM_x     }, //OR 
		{2, 0x08, -1  , false, 8, "OR"       , AM_rm8,AM_r8 , AM_x        }, //OR 
		{2, 0x09, -1  , false, 8, "OR"       , AM_rm32,AM_r32 , AM_x      }, //OR 
		{2, 0x0A, -1  , false, 8, "OR"       , AM_r8,AM_rm8 , AM_x        }, //OR 
		{2, 0x0B, -1  , false, 8, "OR"       , AM_r32,AM_rm32 , AM_x      }, //OR 
		{2, 0xE6, -1  , false,-1, "OUT"      , AM_imm8,AM_AL, AM_x         }, //Output byte in AL to I/O(AM_imm8)
		{2, 0xE7, -1  , false,-1, "OUT"      , AM_imm8,AM_EAX, AM_x        }, //Output dword in EAX to I/O(AM_imm8)
		{1, 0xEE, -1  , false,-1, "OUT"      , AM_DX,AM_AL, AM_x           }, //Output byte in AL to I/O(DX)
		{1, 0xEF, -1  , false,-1, "OUT"      , AM_DX,AM_EAX, AM_x          }, //Output dword in EAX to I/O(DX)
		{1, 0x6E, -1  , false,-1, "OUTSB"     , AM_x,AM_x, AM_x           }, //Output byte from DS:(E)SI to I/O(DX)
		{1, 0x6F, -1  , false,-1, "OUTSD"     , AM_x,AM_x, AM_x          }, //Output dword from DS:(E)SI to I/O (DX)
		{2, 0x66, 0x6F, false,-1, "OUTSW"     , AM_x,AM_x, AM_x          }, //Output word from DS:(E)SI to I/O (DX)
		//{1, 0x8F, -1  , false, 0, "POP"      , m32, AM_x, AM_x             }, //Pop m32
		{1, 0x58, -1  , true ,-1, "POP"      , AM_r32 , AM_x, AM_x            }, //Pop AM_r32
		{1, 0x61, -1  , false,-1, "POPAD"    , AM_x, AM_x, AM_x }, //Pop EDI,... and EAX
		{1, 0x9D, -1  , false,-1, "POPFD"    , AM_x, AM_x, AM_x }, //Pop Stack into EFLAGS Register
		{2, 0xFF, -1  , false, 6, "PUSH"     , AM_rm32 , AM_x, AM_x          }, //Push AM_rm32
		{1, 0x50, -1  , true ,-1, "PUSH"     , AM_r32 , AM_x, AM_x            }, //Push AM_r32
		{2, 0x6A, -1  , false,-1, "PUSH"     , AM_imm8, AM_x, AM_x           }, //Push AM_imm8
		{5, 0x68, -1  , false,-1, "PUSH"     , AM_imm32, AM_x, AM_x          }, //Push AM_imm32
		{1, 0x60, -1  , false,-1, "PUSHAD"   , AM_x, AM_x, AM_x }, //Push All g-regs
		{1, 0x9C, -1  , false,-1, "PUSHFD"   , AM_x, AM_x, AM_x }, //Push EFLAGS
		//{1, 0xD0, -1  , false, 2, "RCL"      , AM_rm8,1, AM_x          }, //Rotate 9 bits left once
		{2, 0xD2, -1  , false, 2, "RCL"      , AM_rm8,AM_CL, AM_x         }, //Rotate 9 bits left CL times
		{3, 0xC0, -1  , false, 2, "RCL"      , AM_rm8,AM_imm8, AM_x      }, //Rotate 9 bits left AM_imm8, AM_xtimes
		//{1, 0xD1, -1  , false, 2, "RCL"      , AM_rm32,1, AM_x         }, //Rotate 33 bits left once
		{2, 0xD3, -1  , false, 2, "RCL"      , AM_rm32,AM_CL, AM_x        }, //Rotate 33 bits left CL times
		{3, 0xC1, -1  , false, 2, "RCL"      , AM_rm32,AM_imm8, AM_x     }, //Rotate 33 bits left AM_imm8, AM_xtimes
		//{1, 0xD0, -1  , false, 3, "RCR"      , AM_rm8,1, AM_x          }, //Rotate 9 bits right once
		{2, 0xD2, -1  , false, 3, "RCR"      , AM_rm8,AM_CL, AM_x         }, //Rotate 9 bits right CL times
		{3, 0xC0, -1  , false, 3, "RCR"      , AM_rm8,AM_imm8, AM_x      }, //Rotate 9 bits right AM_imm8, AM_xtimes
		//{1, 0xD1, -1  , false, 3, "RCR"      , AM_rm32,1, AM_x         }, //Rotate 33 bits right once
		{2, 0xD3, -1  , false, 3, "RCR"      , AM_rm32,AM_CL, AM_x        }, //Rotate 33 bits right CL times
		{3, 0xC1, -1  , false, 3, "RCR"      , AM_rm32,AM_imm8, AM_x     }, //Rotate 33 bits right AM_imm8, AM_xtimes
		//{1, 0xD0, -1  , false, 0, "ROL"      , AM_rm8,1, AM_x          }, //Rotate 8 bits AM_rm8 , AM_xleft once
		{2, 0xD2, -1  , false, 0, "ROL"      , AM_rm8,AM_CL, AM_x         }, //Rotate 8 bits AM_rm8 , AM_xleft CL times
		{3, 0xC0, -1  , false, 0, "ROL"      , AM_rm8,AM_imm8, AM_x      }, //Rotate 8 bits AM_rm8 , AM_xleft AM_imm8, AM_xtimes
		//{1, 0xD1, -1  , false, 0, "ROL"      , AM_rm32,1, AM_x         }, //Rotate 32 bits AM_rm32 , AM_xleft once
		{2, 0xD3, -1  , false, 0, "ROL"      , AM_rm32,AM_CL, AM_x        }, //Rotate 32 bits AM_rm32 , AM_xleft CL times
		{3, 0xC1, -1  , false, 0, "ROL"      , AM_rm32,AM_imm8, AM_x     }, //Rotate 32 bits AM_rm32 , AM_xleft AM_imm8, AM_xtimes
		//{1, 0xD0, -1  , false, 1, "ROR"      , AM_rm8,1 , AM_x         }, //Rotate 8 bits AM_rm8 , AM_xright once
		{2, 0xD2, -1  , false, 1, "ROR"      , AM_rm8,AM_CL, AM_x         }, //Rotate 8 bits AM_rm8 , AM_xright CL times
		{3, 0xC0, -1  , false, 1, "ROR"      , AM_rm8,AM_imm8, AM_x      }, //Rotate 8 bits r/m16 right AM_imm8, AM_xtimes
		//{1, 0xD1, -1  , false, 1, "ROR"      , AM_rm32,1, AM_x         }, //Rotate 32 bits AM_rm32 , AM_xright once
		{2, 0xD3, -1  , false, 1, "ROR"      , AM_rm32,AM_CL, AM_x        }, //Rotate 32 bits AM_rm32 , AM_xright CL times
		{3, 0xC1, -1  , false, 1, "ROR"      , AM_rm32,AM_imm8, AM_x     }, //Rotate 32 bits AM_rm32 , AM_xright AM_imm8, AM_xtimes
		{1, 0xF3, -1  , false,-1, "REP"      , AM_x, AM_x, AM_x }, //Rep prefix
		{1, 0xF3, -1  , false,-1, "REPE"     , AM_x, AM_x, AM_x }, //Rep prefix
		{1, 0xF2, -1  , false,-1, "REPNE"    , AM_x, AM_x, AM_x }, //Rep prefix
		{1, 0x6C, -1  , false,-1, "INSB"     , AM_x, AM_x, AM_x           }, //Input ECX bytes from port DX into ES:[(E)DI]
		{1, 0x6D, -1  , false,-1, "INSD"     , AM_x, AM_x, AM_x          }, //Input ECX dwords from port DX into ES:[(E)DI]
		{2, 0x66, 0x6D, false,-1, "INSW"     , AM_x, AM_x, AM_x          }, //Input ECX dwords from port DX into ES:[(E)DI]
		{1, 0xA4, -1  , false,-1, "MOVSB"    , AM_x, AM_x, AM_x           }, //Move ECX bytes from DS:[(E)SI] to ES:[(E)DI]
		{1, 0xA5, -1  , false,-1, "MOVSD"    , AM_x, AM_x, AM_x         }, //Move ECX dwords from DS:[(E)SI] to ES:[(E)DI]
		{2, 0x66, 0xA5, false,-1, "MOVSW"    , AM_x, AM_x, AM_x         }, //Move ECX dwords from DS:[(E)SI] to ES:[(E)DI]
		{1, 0x6E, -1  , false,-1, "OUTSB"    , AM_x, AM_x, AM_x           }, //Output ECX bytes from DS:[(E)SI] to port DX
		{1, 0x6F, -1  , false,-1, "OUTSD"    , AM_x, AM_x, AM_x          }, //Output ECX dwords from DS:[(E)SI] to port DX
		{2, 0x66, 0x6F, false,-1, "OUTSW"    , AM_x, AM_x, AM_x          }, //Output ECX dwords from DS:[(E)SI] to port DX
		{1, 0xAC, -1  , false,-1, "LODSB"     , AM_x, AM_x, AM_x        }, //Load ECX bytes from DS:[(E)SI] to AL
		{1, 0xAD, -1  , false,-1, "LODSD"     , AM_x, AM_x, AM_x       }, //Load ECX dwords from DS:[(E)SI] to EAX
		{2, 0x66, 0xAD, false,-1, "LODSW"     , AM_x, AM_x, AM_x       }, //Load ECX words from DS:[(E)SI] to AX
		{1, 0xA6, -1  , false,-1, "CMPSB"     , AM_x, AM_x, AM_x           }, //Find non/matching bytes in m and m
		{1, 0xA7, -1  , false,-1, "CMPSD"     , AM_x, AM_x, AM_x         }, //Find non/matching dwords in m and m
		{2, 0x66, 0xA7, false,-1, "CMPSW"     , AM_x, AM_x, AM_x         }, //Find non/matching words in m and m
		{1, 0xAE, -1  , false,-1, "SCASB"     , AM_x, AM_x, AM_x        }, //Find non-AL byte starting at 
		{1, 0xAF, -1  , false,-1, "SCASD"     , AM_x, AM_x, AM_x       }, //Find non-EAX dword starting at 
		{2, 0x66, 0xAF, false,-1, "SCASW"     , AM_x, AM_x, AM_x       }, //Find non-AX word starting at 
		{1, 0xC3, -1  , false,-1, "RET"      , AM_x, AM_x, AM_x }, //Near return 
		//{1, 0xCB, -1  , false,-1, "RET"      , AM_x, AM_x, AM_x }, //Far return 
		{3, 0xC2, -1  , false,-1, "RET"      , AM_imm16, AM_x, AM_x     }, //Near return, pop AM_imm16 bytes from stack
		//{1, 0xCA, -1  , false,-1, "RET"      , AM_imm16, AM_x, AM_x      }, //Far return, pop AM_imm16 bytes from stack
		{1, 0x9E, -1  , false,-1, "SAHF"     , AM_x, AM_x, AM_x }, //Store AH into Flags
		//{1, 0xD0, -1  , false, 4, "SAL"      , AM_rm8,1, AM_x          }, //Shift Arithmetic Left
		{2, 0xD2, -1  , false, 4, "SAL"      , AM_rm8,AM_CL, AM_x         }, //Shift Arithmetic Left
		{3, 0xC0, -1  , false, 4, "SAL"      , AM_rm8,AM_imm8, AM_x      }, //Shift Arithmetic Left
		//{1, 0xD1, -1  , false, 4, "SAL"      , AM_rm32,1, AM_x         }, //Shift Arithmetic Left
		{2, 0xD3, -1  , false, 4, "SAL"      , AM_rm32,AM_CL, AM_x        }, //Shift Arithmetic Left
		{3, 0xC1, -1  , false, 4, "SAL"      , AM_rm32,AM_imm8, AM_x     }, //Shift Arithmetic Left
		//{1, 0xD0, -1  , false, 7, "SAR"      , AM_rm8,1, AM_x          }, //Shift Arithmetic Right
		{1, 0xD2, -1  , false, 7, "SAR"      , AM_rm8,AM_CL, AM_x         }, //Shift Arithmetic Right
		{3, 0xC0, -1  , false, 7, "SAR"      , AM_rm8,AM_imm8, AM_x      }, //Shift Arithmetic Right
		//{1, 0xD1, -1  , false, 7, "SAR"      , AM_rm32,1, AM_x         }, //Shift Arithmetic Right
		{2, 0xD3, -1  , false, 7, "SAR"      , AM_rm32,AM_CL, AM_x        }, //Shift Arithmetic Right
		{3, 0xC1, -1  , false, 7, "SAR"      , AM_rm32,AM_imm8, AM_x     }, //Shift Arithmetic Right
		//{1, 0xD0, -1  , false, 4, "SHL"      , AM_rm8,1, AM_x          }, //Shift Logical Left
		{2, 0xD2, -1  , false, 4, "SHL"      , AM_rm8,AM_CL, AM_x         }, //Shift Logical Left
		{3, 0xC0, -1  , false, 4, "SHL"      , AM_rm8,AM_imm8, AM_x      }, //Shift Logical Left
		//{1, 0xD1, -1  , false, 4, "SHL"      , AM_rm32,1, AM_x         }, //Shift Logical Left
		{2, 0xD3, -1  , false, 4, "SHL"      , AM_rm32,AM_CL, AM_x        }, //Shift Logical Left
		{3, 0xC1, -1  , false, 4, "SHL"      , AM_rm32,AM_imm8, AM_x     }, //Shift Logical Left
		//{1, 0xD0, -1  , false, 5, "SHR"      , AM_rm8,1, AM_x          }, //Shift Logical Right
		{2, 0xD2, -1  , false, 5, "SHR"      , AM_rm8,AM_CL, AM_x         }, //Shift Logical Right
		{3, 0xC0, -1  , false, 5, "SHR"      , AM_rm8,AM_imm8, AM_x      }, //Shift Logical Right
		//{1, 0xD1, -1  , false, 5, "SHR"      , AM_rm32,1, AM_x         }, //Shift Logical Right
		{2, 0xD3, -1  , false, 5, "SHR"      , AM_rm32,AM_CL, AM_x        }, //Shift Logical Right
		{3, 0xC1, -1  , false, 5, "SHR"      , AM_rm32,AM_imm8, AM_x     }, //Shift Logical Right
		{2, 0x1C, -1  , false,-1, "SBB"      , AM_AL,AM_imm8, AM_x        }, //Subtract with borrow 
		{5, 0x1D, -1  , false,-1, "SBB"      , AM_EAX,AM_imm32, AM_x      }, //Subtract with borrow 
		{3, 0x80, -1  , false, 3, "SBB"      , AM_rm8,AM_imm8, AM_x      }, //Subtract with borrow 
		{6, 0x81, -1  , false, 3, "SBB"      , AM_rm32,AM_imm32, AM_x    }, //Subtract with borrow 
		{3, 0x83, -1  , false, 3, "SBB"      , AM_rm32,AM_imm8, AM_x     }, //Subtract with borrow 
		{2, 0x18, -1  , false, 8, "SBB"      , AM_rm8,AM_r8 , AM_x        }, //Subtract with borrow 
		{2, 0x19, -1  , false, 8, "SBB"      , AM_rm32,AM_r32 , AM_x      }, //Subtract with borrow 
		{2, 0x1A, -1  , false, 8, "SBB"      , AM_r8,AM_rm8 , AM_x        }, //Subtract with borrow 
		{2, 0x1B, -1  , false, 8, "SBB"      , AM_r32,AM_rm32 , AM_x      }, //Subtract with borrow 
		{3, 0x0F, 0x97, false, 8, "SETA"     , AM_rm8 , AM_x, AM_x     }, //Set byte if above 
		{3, 0x0F, 0x93, false, 8, "SETAE"    , AM_rm8 , AM_x, AM_x     }, //Set byte if above or equal
		{3, 0x0F, 0x92, false, 8, "SETB"     , AM_rm8 , AM_x, AM_x     }, //Set byte if below 
		{3, 0x0F, 0x96, false, 8, "SETBE"    , AM_rm8 , AM_x, AM_x     }, //Set byte if below or equal 
		{3, 0x0F, 0x94, false, 8, "SETE"     , AM_rm8 , AM_x, AM_x     }, //Set byte if equal 
		{3, 0x0F, 0x9F, false, 8, "SETG"     , AM_rm8 , AM_x, AM_x     }, //Set byte if greater 
		{3, 0x0F, 0x9D, false, 8, "SETGE"    , AM_rm8 , AM_x, AM_x     }, //Set byte if greater or equal
		{3, 0x0F, 0x9C, false, 8, "SETL"     , AM_rm8 , AM_x, AM_x     }, //Set byte if less 
		{3, 0x0F, 0x9E, false, 8, "SETLE"    , AM_rm8 , AM_x, AM_x     }, //Set byte if less or equal 
		{3, 0x0F, 0x95, false, 8, "SETNE"    , AM_rm8 , AM_x, AM_x     }, //Set byte if not equal 
		{3, 0x0F, 0x91, false, 8, "SETNO"    , AM_rm8 , AM_x, AM_x     }, //Set byte if not overflow 
		{3, 0x0F, 0x99, false, 8, "SETNS"    , AM_rm8 , AM_x, AM_x     }, //Set byte if not sign 
		{3, 0x0F, 0x90, false, 8, "SETO"     , AM_rm8 , AM_x, AM_x     }, //Set byte if overflow 
		{3, 0x0F, 0x9A, false, 8, "SETPE"    , AM_rm8 , AM_x, AM_x     }, //Set byte if parity even 
		{3, 0x0F, 0x9B, false, 8, "SETPO"    , AM_rm8 , AM_x, AM_x     }, //Set byte if parity odd 
		{3, 0x0F, 0x98, false, 8, "SETS"     , AM_rm8 , AM_x, AM_x     }, //Set byte if sign 
		{4, 0x0F, 0xA4, false, 8, "SHLD"     , AM_rm32,AM_r32,AM_imm8        }, //Double Precision Shift Left
		{3, 0x0F, 0xA5, false, 8, "SHLD"     , AM_rm32,AM_r32,AM_CL          }, //Double Precision Shift Left
		{4, 0x0F, 0xAC, false, 8, "SHRD"     , AM_rm32,AM_r32,AM_imm8        }, //Double Precision Shift Right
		{3, 0x0F, 0xAD, false, 8, "SHRD"     , AM_rm32,AM_r32,AM_CL          }, //Double Precision Shift Right
		{1, 0xF9, -1  , false,-1, "STC"      , AM_x, AM_x, AM_x }, //Set Carry Flag
		{1, 0xFD, -1  , false,-1, "STD"      , AM_x, AM_x, AM_x }, //Set Direction Flag
		{1, 0xFB, -1  , false,-1, "STI"      , AM_x, AM_x, AM_x }, //Set Interrup Flag
		{1, 0xAA, -1  , false,-1, "STOSB"     , AM_x, AM_x, AM_x        }, //Fill ECX bytes at ES:[(E)DI] with AL
		{1, 0xAB, -1  , false,-1, "STOSD"     , AM_x, AM_x, AM_x       }, //Fill ECX dwords at ES:[(E)DI] with EAX
		{2, 0x66, 0xAB, false,-1, "STOSW"     , AM_x, AM_x, AM_x       }, //Fill ECX words at ES:[(E)DI] with AX
		{2, 0x2C, -1  , false,-1, "SUB"      , AM_AL,AM_imm8, AM_x        }, //Subtract 
		{5, 0x2D, -1  , false,-1, "SUB"      , AM_EAX,AM_imm32, AM_x      }, //Subtract 
		{3, 0x80, -1  , false, 5, "SUB"      , AM_rm8,AM_imm8, AM_x      }, //Subtract 
		{6, 0x81, -1  , false, 5, "SUB"      , AM_rm32,AM_imm32, AM_x    }, //Subtract 
		{3, 0x83, -1  , false, 5, "SUB"      , AM_rm32,AM_imm8, AM_x     }, //Subtract 
		{2, 0x28, -1  , false, 8, "SUB"      , AM_rm8,AM_r8 , AM_x        }, //Subtract 
		{2, 0x29, -1  , false, 8, "SUB"      , AM_rm32,AM_r32 , AM_x      }, //Subtract 
		{2, 0x2A, -1  , false, 8, "SUB"      , AM_r8,AM_rm8 , AM_x        }, //Subtract 
		{2, 0x2B, -1  , false, 8, "SUB"      , AM_r32,AM_rm32 , AM_x      }, //Subtract 
		{2, 0xA8, -1  , false,-1, "TEST"     , AM_AL,AM_imm8, AM_x        }, //Logical Compare
		{5, 0xA9, -1  , false,-1, "TEST"     , AM_EAX,AM_imm32, AM_x      }, //Logical Compare
		{3, 0xF6, -1  , false, 0, "TEST"     , AM_rm8,AM_imm8, AM_x      }, //Logical Compare
		{6, 0xF7, -1  , false, 0, "TEST"     , AM_rm32,AM_imm32, AM_x    }, //Logical Compare
		{2, 0x84, -1  , false, 8, "TEST"     , AM_rm8,AM_r8 , AM_x        }, //Logical Compare
		{2, 0x85, -1  , false, 8, "TEST"     , AM_rm32,AM_r32 , AM_x      }, //Logical Compare
		{2, 0x0F, 0x0B, false,-1, "UD2"      , AM_x, AM_x, AM_x }, //Undifined Instruction
		{1, 0x9B, -1  , false,-1, "WAIT"     , AM_x, AM_x, AM_x }, //Wait
		{3, 0x0F, 0xC0, false, 8, "XADD"     , AM_rm8,AM_r8 , AM_x        }, //Exchange and Add
		{3, 0x0F, 0xC1, false, 8, "XADD"     , AM_rm32,AM_r32 , AM_x      }, //Exchange and Add
		{1, 0x90, -1  , true ,-1, "XCHG"     , AM_EAX,AM_r32 , AM_x        }, //Exchange AM_r32 , AM_xwith EAX
		{1, 0x90, -1  , true ,-1, "XCHG"     , AM_r32,AM_EAX, AM_x         }, //Exchange EAX with AM_r32
		{2, 0x86, -1  , false, 8, "XCHG"     , AM_rm8,AM_r8 , AM_x        }, //Exchange byte 
		{2, 0x86, -1  , false, 8, "XCHG"     , AM_r8,AM_rm8 , AM_x        }, //Exchange byte 
		{2, 0x87, -1  , false, 8, "XCHG"     , AM_rm32,AM_r32 , AM_x      }, //Exchange doubleword 
		{2, 0x87, -1  , false, 8, "XCHG"     , AM_r32,AM_rm32 , AM_x      }, //Exchange doubleword 
		{1, 0xD7, -1  , false,-1, "XLAT"     , AM_x, AM_x, AM_x        }, //Table Look-up Translation
		{2, 0x34, -1  , false,-1, "XOR"      , AM_AL,AM_imm8, AM_x        }, //Logical Exclusive OR
		{5, 0x35, -1  , false,-1, "XOR"      , AM_EAX,AM_imm32, AM_x      }, //Logical Exclusive OR
		{3, 0x80, -1  , false, 6, "XOR"      , AM_rm8,AM_imm8, AM_x      }, //Logical Exclusive OR
		{6, 0x81, -1  , false, 6, "XOR"      , AM_rm32,AM_imm32, AM_x    }, //Logical Exclusive OR
		{3, 0x83, -1  , false, 6, "XOR"      , AM_rm32,AM_imm8, AM_x     }, //Logical Exclusive OR
		{2, 0x30, -1  , false, 8, "XOR"      , AM_rm8,AM_r8 , AM_x        }, //Logical Exclusive OR
		{2, 0x31, -1  , false, 8, "XOR"      , AM_rm32,AM_r32 , AM_x      }, //Logical Exclusive OR
		{2, 0x32, -1  , false, 8, "XOR"      , AM_r8,AM_rm8 , AM_x        }, //Logical Exclusive OR
		{2, 0x33, -1  , false, 8, "XOR"      , AM_r32,AM_rm32 , AM_x      }, //Logical Exclusive OR
		{0, -1, -1, false, -1, "", AM_x, AM_x, AM_x }
	};
