#ifndef X86_H
#define X86_H
#include "misc.h"

enum directives {
  AS_err = 361,  // erroneous opcode
  AS_nul = 362,  // blank opcode
  AS_beg = 363,  // introduce program
  AS_end = 364,  // end of source
  AS_mac = 365,  // introduce macro
  AS_ds  = 366,  // define storage
  AS_equ = 367,  // equate
  AS_org = 368,  // set location counter
  AS_if  = 369,  // conditional
  AS_dc  = 370   // define constant byte
};

typedef enum { AM_x, 
	AM_AL, AM_EAX,  AM_DX, AM_CL,
	AM_imm8, AM_rel8, AM_rm8, AM_r8,
	AM_imm16,
	AM_imm32, AM_rel32, AM_rm32, AM_r32
} ADR_MODE;

struct X86_instr {
   // fields used by the code generator
   int length; // number of bytes not counting prefixes
   int opcode; // if valid then >= 0
   int opcode2; // -1 if not present

   bool plusr; // if true, add reg. number to opcode byte
   int modrm; // -1 if not present, 8 all fields used for operands, 0-7 opcode extension

   // fields used by the syntax analyzer
   char mnemonic[10];
   ADR_MODE am1, am2, am3;
};

class X86
{
public:
	X86();
	int findfirst(char *mnemonic);
	int findnext();
    int illegal(); 
	int null();
	int END();
    static bool hasImm32Opnd( int tmpl );
    static const X86_instr table[];

protected:
	int curr;
};
#endif /*X86_H*/