#ifndef AS_H
#define AS_H

#include "misc.h"
#include "sa.h"
#include "st.h"
#include "sh.h"

#define MAX_LOC 65536


class AS {
  public:
    void assemble(bool &errors);
    // Assembles and lists program.
    // Assembled code is dumped to file for later interpretation, and left
    // in pseudo-machine memory for immediate interpretation if desired.
    // Returns errors = true if assembly fails

    AS(char *sourcename, char *listname, char *version, char* objfilename);
    // Instantiates version of the assembler to process sourcename, creating
    // listings in listname, and generating code for associated machine M

  private:
    SH *Srce;
    LA *Lex;
    SA *Parser;
    ST *Table;
    
	struct objlines { int location, 
		sprefix, 
		opcode, opcode2, 
		modrm, sib, 
		immoffs_size /* in bytes, for immoffs field */, 
		immoffs; };

    objlines objline;    // current line as assembled
    int location;   // location counter
    bool assembling;     // monitor progress of assembly

	void evaluate(ADR_MODE am, SA_addresses address, int &dsize, int &value, bool &error);
    void listcode(void);
    void listsourceline(SA_unpackedlines &srcline, bool coderequired);
    void assembleline(SA_unpackedlines &srcline, objlines &objline,
                      bool &failure);
    void firstpass(void);
    void secondpass(bool &errors);
	void saveobjcode(void);

    char WorkFileName[256];
	char objname[256];
    FILE *workfile;      // intermediate file
	unsigned char mem[MAX_LOC];
};

#endif /*AS_H*/
